"""This module keeps all the base classes for mad_gui.plot_tools."""
import pandas as pd
import pyqtgraph as pg
from pyqtgraph.GraphicsScene.mouseEvents import MouseClickEvent
from PySide2.QtCore import Slot
from PySide2.QtGui import QColor, QCursor, QMouseEvent, QPalette

from mad_gui.config import Config
from mad_gui.models.local import AnnotationData, PlotData
from mad_gui.plot_tools.labels import SynchronizationLabel
from mad_gui.plot_tools.labels.base_label import BaseEventLabel, BaseRegionLabel
from typing import List, Optional, Type, Union


class BasePlot(pg.PlotWidget):
    """A base plot that can display data and additionally show a yellow line to indicate the current position in a
    video."""

    def __init__(
        self,
        plot_data: PlotData = None,
        initial_plot_channels=None,
        label_classes: List[BaseRegionLabel] = None,
        event_classes: Optional[List[BaseEventLabel]] = None,
        parent=None,
    ):
        super().__init__(parent=None)
        self.parent = parent
        self.plot_data = plot_data
        self.label_classes = label_classes
        self.event_classes = event_classes
        plotted_data = getattr(plot_data, "data", None)
        if plotted_data is not None:
            columns = plotted_data.columns
        else:
            columns = None
        self.initial_plot_channels = initial_plot_channels or columns
        self.configure_style()
        self.video_cursor_line = None
        self.cursor_line_pen = pg.mkPen(color="y", width=1)
        self.sync_item = None
        self.sync_info = None
        self.label_ranges = None
        self.event_ranges = None
        self.event_labels = {}
        self._initialize_labels(label_classes)
        self._initialize_events(event_classes)

    def _initialize_events(self, event_classes: List):
        # if not hasattr(self.plot_data.annotations, "events"):
        #    return
        # df = self.plot_data.annotations["events"].data
        # for _, event in df.iterrows():
        #    pos = self.snap_to_sample(event.pos / self.plot_data.sampling_rate_hz)
        #    self.addItem(BaseEventLabel(pos=pos, span=(event.min_height, event.max_height), parent=self))
        if event_classes is None:
            return
        event_ranges = pd.DataFrame()
        for event_class in event_classes:
            if event_class.name not in self.plot_data.annotations.keys():
                self.plot_data.annotations[event_class.name] = AnnotationData()
            self.set_events(event_class, self.plot_data.annotations[event_class.name].data)

            event_range = pd.DataFrame(
                index=[event_class.name],
                data=[[event_class.min_height, event_class.max_height]],
                columns=["min_height", "max_height"],
            )
            event_ranges = event_ranges.append(event_range)
        self.event_ranges = event_ranges

    def _initialize_labels(self, labels: List):
        label_ranges = pd.DataFrame()

        for label_class in labels:
            if hasattr(self.plot_data, "annotations"):
                self._ensure_annotations_available(label_class)
                self.set_labels(label_class, self.plot_data.annotations[label_class.name].data)

            label_range = pd.DataFrame(
                index=[label_class.name],
                data=[[label_class.min_height, label_class.max_height]],
                columns=["min_height", "max_height"],
            )
            label_ranges = label_ranges.append(label_range)
        self.label_ranges = label_ranges

    def _ensure_annotations_available(self, label_class: Union[BaseRegionLabel, BaseEventLabel]):
        if label_class.name not in self.plot_data.annotations.keys():
            self.plot_data.annotations[label_class.name] = AnnotationData()

    @Slot(BaseEventLabel, pd.DataFrame)
    def set_events(self, label_class: Type[BaseEventLabel], df: pd.DataFrame):
        if df is None or df.empty:
            return
        self.clear_labels(label_class)
        for _, event in df.iterrows():
            new_event = label_class(
                pos=event.pos,
                description=event.description,
                parent=self,
            )
            self.addItem(new_event)

    @Slot(BaseRegionLabel, pd.DataFrame)
    def set_labels(self, label_class: Type[BaseRegionLabel], df: pd.DataFrame):
        if df is None or df.empty:
            return
        self.clear_labels(label_class)
        for _, activity in df.iterrows():
            # make sure there are no np.nans in any string field
            mask = activity.index.isin(["start", "end"])
            activity[~mask] = activity[~mask].fillna("")

            activity = self._enforce_columns(activity, necessary_columns=["identifier", "description", "events"])

            if hasattr(label_class, "events"):
                events = activity[label_class.events]
                plot_events = getattr(label_class, "plot_events", events.keys())
            else:
                events = None
            new_activity = label_class(
                identifier=activity.identifier,
                description=activity.description,
                start=activity.start,
                end=activity.end,
                events=events,
                parent=self,
            )
            self.addItem(new_activity)

            for event_name, event in new_activity.event_labels.items():
                if event_name in plot_events:
                    self.addItem(event)

    @staticmethod
    def _enforce_columns(activity: pd.Series, necessary_columns: List) -> pd.Series:
        # make sure all required fields are available
        for parameter in necessary_columns:
            if parameter not in activity.index:
                activity = activity.append(pd.Series(data=[None], index=[parameter]))
        return activity

    def set_title(self, title: str):
        """Set the title, which will be shown centered on the top of the plot.

        Parameters
        ----------
        title
            The title that should be shown on the top of the plot.

        """
        self.setTitle(title)
        self.plotItem.titleLabel.setText(text=title, color=Config.theme.FAU_COLORS["dark_blue"])

    def configure_style(self):
        bg_color = Config.theme.PLOT_BACKGROUND
        if bg_color:
            self.setBackground(bg_color)
            for i_channel in ["bottom", "left"]:
                self._adapt_channel_color(i_channel, self.parent.palette().color(QPalette.Active, QPalette.Window))

    def _adapt_channel_color(self, channel: str, color: QColor):
        """Make channel color in FAU style"""
        ax = self.getAxis(channel)
        ax.setPen(color=color, width=1)
        ax.setTextPen(color=color)

    def set_coupled_plot(self, other: Optional[pg.PlotWidget] = None):
        """Couple another plot to this one, such that both plots always have the same x- and y- limits.

        Parameters
        ----------
        other
            Usually another object of :class:`~mad_gui.plot_tools.SensorPlot`.
        """
        if getattr(Config.settings, "SENSORS_SYNCHRONIZED", False):
            self.setXLink(other)
        if getattr(Config.settings, "BIND_Y_AXIS", False):
            self.setYLink(other)

    def enterEvent(self, ev):  # noqa
        # Camelcase method overwrites qt method
        self.setFocus()
        return super().enterEvent(ev)

    def inside_plot_range(self, local_pos):
        x_min = self.viewRange()[0][0]
        x_max = self.viewRange()[0][1]
        y_min = self.viewRange()[1][0]
        y_max = self.viewRange()[1][1]
        if x_min < local_pos.x() < x_max and y_min < local_pos.y() < y_max:
            return True
        return False

    def _inside_range(self, pos, label_ranges: pd.DataFrame):
        for label_name, label_range in label_ranges.iterrows():

            y_min = self.viewRange()[1][0]
            y_max = self.viewRange()[1][1]

            if label_range.min_height <= (pos.y() - y_min) / (y_max - y_min) <= label_range.max_height:
                return label_name
        return None

    def inside_event_range(self, pos):
        if self.event_ranges.empty:
            return None
        return self._get_event_class(self._inside_range(pos, self.event_ranges))

    def inside_label_range(self, pos):
        return self._get_label_class(self._inside_range(pos, self.label_ranges))

    def _get_event_class(self, event_name: str):
        for label in self.event_classes:
            if label.name == event_name:
                return label
        return None

    def _get_label_class(self, label_name: str):
        for label in self.label_classes:
            if label.name == label_name:
                return label
        return None

    def get_mouse_pos_from_event(self, ev):
        if isinstance(ev, QMouseEvent):
            position = ev.localPos()
        elif isinstance(ev, MouseClickEvent):
            position = ev.scenePos()
        else:
            # Keyboard events or others
            position = self.mapFromGlobal(QCursor.pos())
        return self.plotItem.vb.mapSceneToView(position)

    def add_video_cursor_line(self, pos: int = None):
        """Add a line that shows to which sample in the signal the currently shown video frame corresponds.

        Parameters
        ----------
        pos
            Position at which the line should be shown (in seconds)
        """
        if pos is None:
            x_range = self.viewRange()[0][1] - self.viewRange()[0][0]
            pos = self.viewRange()[0][0] + x_range / 2
        if not self.video_cursor_line:
            self.video_cursor_line = pg.InfiniteLine(pos=pos, angle=90, name="vertical_line", pen=self.cursor_line_pen)
            self.addItem(self.video_cursor_line)

    def remove_video_cursor_line(self):
        """Remove the line that indicates video-signal correspondence when the video window is closed."""
        if self.video_cursor_line:
            self.removeItem(self.video_cursor_line)
            self.video_cursor_line = None

    def snap_to_sample(self, pos: float):
        sampling_rate_hz = self.plot_data.sampling_rate_hz
        # make at least sure it is at the position of an actual sample
        return round(pos * sampling_rate_hz) / sampling_rate_hz

    def _percent_to_position(self, percent_since_start: float):
        if self.plot_data.data is None:
            return 0
        if self.sync_info is None or any(pd.isna(self.sync_info)):
            x_axis = self.plotItem.listDataItems()[0].getData()[0]
            sec = percent_since_start / 100 * x_axis[-1]
        else:
            stream_length = (self.sync_info["end"] - self.sync_info["start"]) / self.plot_data.sampling_rate_hz
            sec = self.sync_info["start"] / self.plot_data.sampling_rate_hz + stream_length * percent_since_start / 100
        return sec

    def move_video_cursor_line(self, percent_since_start: float):
        """Move the line that indicates to which signal sample the current video frame corresponds.

        Parameters
        ----------
        percent_since_start
            the percentage of the data stream to jump to since start
        """
        if self.video_cursor_line:  # otherwise this causes an error during testing
            sec = self._percent_to_position(percent_since_start)
            self.video_cursor_line.setValue(sec)

    @Slot(int)
    def set_graph_position(self, percent_since_start: float):
        """Move the graph such that `sample` is in the middle of the plot.

        Parameters
        ----------
        percent_since_start
            the percentage of the data stream to jump to since start

        """
        if self.state.mode == "sync":
            return
        sec = self._percent_to_position(percent_since_start)
        x_min = sec - getattr(Config.settings, "PLOT_WIDTH_PLAYING_VIDEO", 20) * 0.5
        x_max = sec + getattr(Config.settings, "PLOT_WIDTH_PLAYING_VIDEO", 20) * 0.5
        self.setXRange(x_min, x_max)

    def finish_syncing(self):
        if self.sync_item:
            start_sample = self.sync_item.getRegion()[0] * self.plot_data.sampling_rate_hz
            end_sample = self.sync_item.getRegion()[1] * self.plot_data.sampling_rate_hz
            self.sync_info = pd.Series(data=[start_sample, end_sample], index=["start", "end"])
            self._remove_sync_item()
        self.add_video_cursor_line()

    def add_sync_item(self):
        if self.sync_item:
            self.autoRange()
            return
        if self.plot_data.data is None:
            return
        if self.sync_info is None:
            start = self.viewRange()[0][0]
            end = self.viewRange()[0][1]
            x_range = end - start
            self.sync_item = SynchronizationLabel(
                start=(start + 0.1 * x_range) * self.plot_data.sampling_rate_hz,
                end=(end - 0.1 * x_range) * self.plot_data.sampling_rate_hz,
                parent=self,
            )
        else:
            self.sync_item = SynchronizationLabel(
                start=self.sync_info.start,
                end=self.sync_info.end,
                parent=self,
            )
        self.addItem(self.sync_item)
        self.autoRange()

    def _remove_sync_item(self):
        self.removeItem(self.sync_item)
        self.sync_item = None



# pylint: skip-file
"""The :py:mod:`mad_gui.plugins` keeps files that tell the GUI how to interact with data from different sources.

If data of a specific system should be used within our GUI, a file in this directory needs to implement the
functionalities given by :py:mod:`mad_gui.plugins.base`, see classes below.
"""

from mad_gui.plugins.base import BaseExporter, BaseImporter, BaseAlgorithm
from mad_gui.plugins.example import ExampleImporter, ExampleExporter

__all__ = ["BaseImporter", "BaseAlgorithm", "BaseExporter", "ExampleImporter", "ExampleExporter"]



"""Select which kind of data should be loaded/saved."""

from PySide2.QtCore import Qt
from PySide2.QtWidgets import QCheckBox, QDialog, QDialogButtonBox, QVBoxLayout

from mad_gui.state_keeper import StateKeeper


class DataSelector(QDialog):
    def __init__(self, parent=None, labels=None):
        super().__init__(parent)
        self.parent = parent
        self.setWindowModality(Qt.ApplicationModal)
        self.setPalette(self.parent.palette())
        self.vbox = QVBoxLayout()
        self.labels = labels
        self.add_checkboxes_to_vbox()
        self.add_ok_btn()
        self.setLayout(self.vbox)
        self.setWindowTitle("Select data types")

    def add_checkboxes_to_vbox(self):
        self.boxes = {}
        self.boxes["sensor_data"] = QCheckBox("Sensor data")
        self.boxes["sensor_data"].setCheckState(Qt.Checked)
        self.boxes["sensor_data"].setPalette(self.palette())
        self.vbox.addWidget(self.boxes["sensor_data"])
        for label in self.labels:
            self.boxes[label] = QCheckBox(label)
            self.boxes[label].setCheckState(Qt.Checked)
            self.boxes[label].setPalette(self.palette())
            self.vbox.addWidget(self.boxes[label])

    def add_ok_btn(self):
        self.ok_btn = QDialogButtonBox(QDialogButtonBox.Ok)
        button_style = self.parent.ui.btn_add_label.styleSheet()
        self.ok_btn.setStyleSheet(button_style)
        self.ok_btn.clicked.connect(self.handle_ok_click)
        self.vbox.addWidget(self.ok_btn)

    def handle_ok_click(self):
        data_to_load = {k: v.isChecked() for k, v in self.boxes.items()}
        #    "sensor": self.boxes["sensor"].isChecked(),
        #    "activities": self.box_activities.isChecked(),
        #    "strides": self.box_strides.isChecked(),
        # }
        StateKeeper.announce_data_types.emit(data_to_load)
        self.close()

    def ask_user(self):
        self.exec_()



from mad_gui.plot_tools.plots.base_plot import BasePlot
from mad_gui.plot_tools.plots.video_plot import VideoPlot
from mad_gui.plot_tools.plots.sensor_plot import SensorPlot

__all__ = [
    "BasePlot",
    "SensorPlot",
    "VideoPlot",
]



from __future__ import annotations

from pyqtgraph import InfiniteLine, mkPen
from PySide2.QtCore import QEvent, Qt
from PySide2.QtGui import QMouseEvent

from mad_gui.components.dialogs import NestedLabelSelectDialog
from mad_gui.components.dialogs.user_information import UserInformation
from mad_gui.config import Config
from mad_gui.plot_tools.labels import PartialLabel
from mad_gui.plot_tools.labels.base_label import BaseEventLabel, BaseRegionLabel, InvalidStartEnd, NoLabelSelected
from mad_gui.plot_tools.plots.base_mode_handler import BaseModeHandler
from typing import Optional


class AddModeHandler(BaseModeHandler):
    mode = "add"
    _partial_label: Optional[PartialLabel] = None
    _potential_start: Optional[InfiniteLine] = None

    def __init__(self, sensor_plot):
        super().__init__(plot=sensor_plot)

    def handle_mouse_click(self, ev, pos=None):  # noqa
        if ev.button() == Qt.MouseButton.RightButton:
            super().handle_mouse_click(ev)
            return
        if pos:
            mouse_position = pos
        else:
            mouse_position = self.plot.get_mouse_pos_from_event(ev)
        if self.plot.inside_label_range(mouse_position):
            if ev.modifiers() == Qt.ControlModifier:
                self._add_event_at_mouse_pos(mouse_position)
            else:
                self._add_label_at_mouse_pos(mouse_position)
        elif self.plot.inside_event_range(mouse_position) and ev.modifiers() == Qt.ControlModifier:
            self._add_event_at_mouse_pos(mouse_position)
        else:
            super().handle_mouse_click(ev)
        if ev.modifiers() == Qt.ShiftModifier:
            # If we press Shift+Mouse we directly create a new event if one is finished
            self._add_label_at_mouse_pos(mouse_position)
        ev.accept()

    def handle_key_press(self, ev):
        """In add mode we handle spacebar and esc.

        If spacebar is pressed we add a new label or finalize it (like with mouse click.
        Esc removes a label that is curretnly added
        """
        key = ev.key()
        if key == Qt.Key_Space:
            local = self.plot.get_mouse_pos_from_event(ev)
            e = QMouseEvent(
                QEvent.MouseButtonPress,
                local,
                local,
                local,
                Qt.MouseButton.LeftButton,
                Qt.MouseButton.LeftButton,
                ev.modifiers(),
                Qt.MouseEventSource.MouseEventSynthesizedByApplication,
            )
            self.handle_mouse_click(e, pos=local)
            ev.accept()
            e.accept()
        ev.accept()

    def handle_mouse_movement(self, ev):
        if not self._active:
            return
        if self._partial_label is None or not self.plot.inside_plot_range(self.plot.get_mouse_pos_from_event(ev)):
            self._reposition_potential_start(ev)
            super(AddModeHandler, self).handle_mouse_movement(ev)  # pylint: disable=super-with-arguments
            return
        pos = self.plot.get_mouse_pos_from_event(ev)
        self._reposition_new_end(pos)
        ev.accept()

    def deactivate(self):
        self._active = False
        self._clear_partial_label()

    def _add_event_to_region(self, position: float):
        # self._partial_label
        label_class = self.plot.inside_label_range(position)
        description = NestedLabelSelectDialog(parent=self.plot.parent.parent).get_label(label_class.event_descriptions)
        new_event = BaseEventLabel(
            parent=self.plot,
            pos=position * self.plot.plot_data.sampling_rate_hz,
            min_height=label_class.min_height,
            max_height=label_class.max_height,
            description=description,
        )
        self._partial_label.events.append(new_event)
        self.plot.addItem(new_event)

    def _add_event_at_mouse_pos(self, pos):
        plot = self.plot
        if self._partial_label:
            self._add_event_to_region(pos)
            return
        event_class = plot.inside_event_range(pos)
        if not event_class:
            return
        if getattr(event_class, "snap_to_min", False):
            position = self.plot.snap_to_min(pos.x())
        elif getattr(event_class, "snap_to_max", False):
            position = self.plot.snap_to_max(pos.x())
        else:
            position = self.plot.snap_to_sample(pos.x())

        description = NestedLabelSelectDialog(parent=self.plot.parent).get_label(event_class.descriptions)

        new_event = event_class(
            parent=self.plot,
            pos=position * self.plot.plot_data.sampling_rate_hz,
            min_height=event_class.min_height,
            max_height=event_class.max_height,
            description=description,
        )

        plot.addItem(new_event)

    def _add_label_at_mouse_pos(self, pos):
        plot = self.plot
        # is called when in "Add Label" Mode and user clicks mouse
        label_class = plot.inside_label_range(pos)
        if not label_class:
            return
        if self._partial_label is not None and self._partial_label.label_class is label_class:
            self._finalize_new_label()
            return

        # In case we started a label of another type, we clean it just to be sure
        # TODO: This might be a dumb idea, as this will delete your partial label, if you click in the wrong region
        self._clear_partial_label()
        self._start_new_label(pos)

    def _start_new_label(self, pos):
        plot = self.plot
        label_class = self.plot.inside_label_range(pos)
        if getattr(label_class, "snap_to_min", False) and getattr(label_class, "snap_to_max", False):
            UserInformation.inform(
                f"The class {label_class} is configured to snap to minimum AND maximum. Please "
                f"only set one of those to attributes to `True`. Snapping is deactivated for now."
            )
        elif getattr(label_class, "snap_to_min", False):
            post_process = plot.snap_to_min
        elif getattr(label_class, "snap_to_max", False):
            post_process = plot.snap_to_max
        else:
            post_process = plot.snap_to_sample

        if self._partial_label:
            self._clear_partial_label()
        self._partial_label = PartialLabel(
            raw_start=pos.x(),
            sampling_rate_hz=plot.plot_data.sampling_rate_hz,
            label_class=label_class,
            label_id=plot._get_appropriate_stride_id(),
            label_parent=plot,
            post_process=post_process,
        )
        plot.addItem(self._partial_label)

    def _finalize_new_label(self):
        plot = self.plot
        if self._partial_label is None:
            raise ValueError("No label to finish")
        try:
            new_label = self._partial_label.finalize()
        except InvalidStartEnd:
            UserInformation(parent=plot.parent).inform("New Label has length 0. No new label added.")
            return
        except NoLabelSelected:
            # In cas ethe user opened the label selector, but closed it again, we assume that opening was an accident
            # and allow the user to further modify the label
            return
        for event in new_label.event_labels.values():
            plot.addItem(event)
        plot.addItem(new_label)
        self._clear_partial_label()

    def _reposition_new_end(self, pos):
        pos = self.plot.snap_to_sample(pos.x())
        self._partial_label.update_end(pos)

    def _reposition_potential_start(self, ev):
        pos = self.plot.get_mouse_pos_from_event(ev)
        snapped_pos = self.plot.snap_to_sample(pos.x())
        if self._potential_start:
            self.plot.removeItem(self._potential_start)

        label_class = self.plot.inside_label_range(pos)
        if label_class and not ev.modifiers() == Qt.ControlModifier:
            self._potential_start = InfiniteLine(snapped_pos, pen=mkPen(0, 255, 0, 150, width=2))
            self._potential_start.span = (
                label_class.min_height,
                label_class.max_height,
            )
            self.plot.addItem(self._potential_start)
            return
        event_class = self.plot.inside_event_range(pos)
        if event_class and ev.modifiers() == Qt.ControlModifier:
            self._potential_start = InfiniteLine(snapped_pos, pen=mkPen(255, 0, 255, 150, width=2, style=Qt.DashLine))
            self._potential_start.span = (
                event_class.min_height,
                event_class.max_height,
            )
            self.plot.addItem(self._potential_start)

    def _clear_partial_label(self):
        if self._potential_start is not None:
            self.plot.delete_item(self._potential_start)
            self._potential_start = None
        if self._partial_label is not None:
            for event in self._partial_label.events:
                self.plot.delete_item(event)
            self.plot.delete_item(self._partial_label)
            self._partial_label = None


class InvestigateModeHandler(BaseModeHandler):
    mode = "investigate"

    def __init__(self, sensor_plot):
        super().__init__(plot=sensor_plot)

    def deactivate(self):
        """We do not need to do anything here, since everything happens through activating the other mode."""


class EditModeHandler(BaseModeHandler):
    def __init__(self, sensor_plot):
        super().__init__(sensor_plot)

        for item in self.plot.items():
            if isinstance(item, (BaseRegionLabel, BaseEventLabel)):
                item.make_editable()

    def deactivate(self):
        for item in self.plot.items():
            if isinstance(item, (BaseRegionLabel, BaseEventLabel)):
                item.make_readonly()
        # in case the mouse is hovering over a label, we still have the pointing hand cursor
        self.plot.setCursor(Qt.ArrowCursor)


class RemoveModeHandler(BaseModeHandler):
    def __init__(self, sensor_plot):
        super().__init__(sensor_plot)

        for item in self.plot.items():
            if isinstance(item, (BaseRegionLabel, BaseEventLabel)):
                item.make_removable()

    def deactivate(self):
        for item in self.plot.items():
            if isinstance(item, (BaseRegionLabel, BaseEventLabel)):
                item.make_readonly()


class SyncModeHandler(BaseModeHandler):
    def __init__(self, sensor_plot):
        super().__init__(sensor_plot)
        self.plot.remove_video_cursor_line()
        sensors_synced = getattr(Config.settings, "SENSORS_SYNCHRONIZED", False)
        if self.plot.is_main_plot or not sensors_synced:
            self.plot.add_sync_item()
        if not getattr(Config.settings, "SENSORS_SYNCHRONIZED", False):
            self.plot.set_coupled_plot(None)

        for item in self.plot.items():
            if isinstance(item, BaseRegionLabel):
                item.make_editable()

    def deactivate(self):
        self.plot.finish_syncing()
        for item in self.plot.items():
            if isinstance(item, BaseRegionLabel):
                item.make_readonly()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mad_gui',
 'mad_gui.components',
 'mad_gui.components.dialogs',
 'mad_gui.components.dialogs.plugin_selection',
 'mad_gui.config',
 'mad_gui.models',
 'mad_gui.models.local',
 'mad_gui.plot_tools.labels',
 'mad_gui.plot_tools.plots',
 'mad_gui.plugins',
 'mad_gui.qt_designer',
 'mad_gui.utils',
 'mad_gui.windows']

package_data = \
{'': ['*'], 'mad_gui.qt_designer': ['images/*']}

install_requires = \
['PySide2==5.15.1',
 'pandas',
 'pyqtgraph==0.11.0',
 'python-vlc>=3.0.16120,<4.0.0',
 'sphinx-qt-documentation>=0.3,<0.4',
 'typing-extensions>=3.10.0,<4.0.0']

entry_points = \
{'console_scripts': ['mad-gui = mad_gui:start_gui']}

setup_kwargs = {
    'name': 'mad-gui',
    'version': '1.0.1a0',
    'description': 'Python GUI for annotating and processing time series data.',
    'long_description': '# MaD GUI\n**M**achine Learning \n**a**nd \n**D**ata Analytics \n**G**raphical \n**U**ser \n**I**nterface\n\n[![Test and Lint](https://github.com/mad-lab-fau/mad-gui/workflows/Test%20and%20Lint/badge.svg)](https://github.com/mad-lab-fau/mad-gui/actions/workflows/test_and_lint.yml)\n[![Documentation Status](https://readthedocs.org/projects/mad-gui/badge/?version=latest)](https://mad-gui.readthedocs.io/en/latest/?badge=latest)\n\n\n[![PyPI version shields.io](https://img.shields.io/pypi/v/mad-gui)](https://pypi.org/project/mad-gui/)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/mad-gui)\n\n![GitHub all releases](https://img.shields.io/github/downloads/mad-lab-fau/mad-gui/total?style=social)\n\n## What is it?\nThe MaD GUI is a framework for processing time series data. Its use-cases include visualization, annotation (manual or automated), and algorithmic processing of visualized data and annotations. More information:\n\n - [Documentation](https://mad-gui.readthedocs.io/en/latest/README.html) \n - [Github Repository](https://github.com/mad-lab-fau/mad-gui)\n - [YouTube Playlist](https://www.youtube.com/watch?v=cSFFSTUM4e0&list=PLf4GpKhBjGcswKIkNeahNt5nDxt8oXPue)\n\n## Using our example\n\nIn a python 3.8 environment, execute the following commands or use the section [Development installation](https://github.com/mad-lab-fau/mad-gui#development-installation):\n```\npip install mad_gui\nmad-gui\n```\n\nThis is just to get a first feeling of how the GUI looks like, you can test with our example data:\nYou can [download our example data](https://github.com/mad-lab-fau/mad-gui#example-data) to\ntest our built-in exemplary importer, exemplary algorithms and exemplary label. \nTo see how to open our example data within the GUI, please refer to our section about the \n[User Interface](https://github.com/mad-lab-fau/mad-gui#user-interface).\n\n## Using your data / algorithms\n\nVery short, you will create and inject plugins, similar to this:\n\n```\nfrom mad_gui import start_gui\nfrom my_plugin_package imoprt MyAlgorithm\n\nstart_gui(plugins=MyAlgorithm)\n```\n\nFor more information on how to create your plugins, refer to [our readme](https://github.com/mad-lab-fau/mad-gui#developing-plugins) or our [more extensive documentation](https://mad-gui.readthedocs.io/en/latest/plugin_importer.html).\n',
    'author': 'Malte Ollenschlaeger',
    'author_email': 'malte.ollenschlaeger@fau.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mad-lab-fau/mad-gui',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)



"""Base class for importing and processing sensor data and annotations."""
import abc
import datetime
from pathlib import Path

import pandas as pd

from mad_gui.components.dialogs.user_information import UserInformation
from mad_gui.models.local import PlotData
from typing import Dict, Union


class BasePlugin:
    """All plugins inherit from this."""

    def __init__(self, parent=None):
        """Set a parent, in case this would be necessary at any later stage.

        Parameters
        ----------
        parent
            This could for example be the main window of the GUI, in case the plugin wants to access something there.
            However, this is unlikely to be necessary since plugins receive either the single
            :class:`mad_gui.models.GlobalData` or its attribute `PlotData`, which should be sufficient to do everything.
        """
        self.parent = parent

    @classmethod
    def name(cls) -> str:
        """Return a name, which is used to represent this plugin in a dropdown in the GUI."""
        raise NotImplementedError()


class BaseImporter(BasePlugin):
    """Classes based on this one enable the GUI to load data from different systems/formats.

    Attributes
    ----------
    file_type
        Can restrict the format of the data/video/annotation file that can be loaded with this importer.
    """

    file_type = {"data_file": "*.*", "video_file": "*.*", "annoatation_file": "*.*"}

    @classmethod
    @abc.abstractmethod
    def name(cls) -> str:
        """Return a name, which is used to represent this Importer in a dropdown in the GUI."""
        raise NotImplementedError()

    def load_sensor_data(self, file: str):  # noqa
        """Loading sensor data as it is usually stored by your recording device

        Parameters
        ----------
        file
            Full path to the file that should be used to load data. You will automatically receive this from the GUI
            which calls this importer.

        Returns
        -------
        sensor_data
            A dictionary with one key per sensor. Each of those, again keeps a dictionary, with at least two keys:
            `sensor data` and `sampling_rate_hz`. Behind the key `sensor_data` is a pd.DataFrame with one column per
            channel, `sampling_rate_hz` is a float.
            If this dictionary has further keys, those will later be stored in
            :class:`mad_gui.models.local.PlotData`'s additional_data.

        Examples
        --------
        >>> data = load_sensor_data("/some/file.format")
        >>> data.keys()
        dict_keys(['left_sensor', 'right_sensor'])
        >>> data['left_sensor'].keys()
        dict_keys(['sensor_data', 'sampling_rate_hz'])
        >>> data['left_sensor']['sensor_data']
            acc_x    'acc_y'
        0    9.81       0.1
        1    9.80       0.0
        2    9.82       0.1

        """
        UserInformation.inform(
            "The functionality of loading sensor data is not implemented for the chosen importer / recording system."
        )
        raise NotImplementedError()

    def load_annotations(self, file_path: Union[Path, str]) -> Dict[str, pd.DataFrame]:  # noqa
        """This loads annotations from file_path and converts them into the format for the GUI.

        This method is called by the :class:`~mad_gui.components.dialogs.plugin_selection.LoadDataDialog` in case the
        user selects a file using the "Select annotation" button. In that case, this method should open the file
        specified by `file_path` and then create.

        NOTE: Your importer does not have to implement this method in case you do not want to deliver such
        functionality to the user.

        Parameters
        ----------
        file_path
            file_path to load the annotation data from

        Returns
        -------
        dict
            Dictionary with keys being the titles of the plots in the main window.
            Each of them keeps a pd.DataFrame, which at least has the columns `start` and `end`.

        Examples
        --------
        >>> annotations = load_annotations("/some/file.format")
        >>> annotations.keys()
        dict_keys(['left_sensor', 'right_sensor'])
        >>>annotations['left_sensor']
            identifier    start    end    description
        0            0      100    105   (gait, slow)
        1            1      105    108   (gait, fast)
        2            2      120    130     (standing)
        """
        UserInformation.inform(
            f"The functionality of loading annotations is not implemented for the chosen importer / recording system "
            f"({self.name()})."
        )
        raise NotImplementedError()

    def get_start_time(self, *args, **kwargs) -> datetime.time:  # noqa
        """Get the start time of the corresponding data.

        This may be used by :class:`mad_gui.plot_tools.plots.SensorPlot` to set the channel labels. However, you do not
        necessarily have to implement it. If you do not implement it, the x-channel will simply start at 0 seconds.

        NOTE: Your importer does not have to implement this method. If you do not implement it, the first sample of
        the shown data will be displayed at second 0 and the x-channel is going to be in seconds.

        Parameters
        ----------
        sensor_data_path
            The path of the sensor data that should be displayed in :class:`mad_gui.windows.MainWindow`
        """
        # Implementing this for backward compa
        return None

    @staticmethod
    def get_sync_file(video_file: str) -> str:
        """Searches for an excel file that has `sync` in its name and returns the file name.

        Attributes
        ----------
        video_file
            The path of the video file that is being shown.

        Returns
        -------
        sync_file
            The path of the excel file that has `sync` in its name and returns the sync indices from there.
        """
        files = list(Path(video_file).parent.glob("*sync*.xlsx"))
        if len(files) == 0:
            UserInformation.inform(
                text="Video and data not synchronized because not sync file was found.",
                help_link="https://mad-gui.readthedocs.io/en/latest/troubleshooting.html#video-and"
                "-data-not-synchronized",
            )
            return ""
        files = [file for file in files if not file.name.startswith("~")]
        if len(files) == 1:
            return files[0]
        UserInformation.inform(
            text="Please make sure there is exactly one file that contains `sync` in its name and "
            "ends with "
            "`.xlsx`."
        )
        return ""

    @classmethod
    def get_video_signal_synchronization(cls, sync_file: str) -> pd.DataFrame:  # noqa
        """Get information regarding sync for video and signal."

        The Excel file should have as first column (index) "start" and "end" and the columns should be
        "PLOTNAME_sample" video_ms."

        Attributes
        ----------
        sync_file
            The path of the sync file for the video that is being shown.

        Returns
        -------
        sync_indices
            A dataframe which tells the gui which frames of the video correspond to which samples in the signal.
            Currently only implemented to accept two synchronized events. Between those, the GUI interpolates linearly.
        """
        try:
            sync = pd.read_excel(sync_file, index_col=0, engine="openpyxl")
            return sync
        except IndexError:
            UserInformation.inform("Format of the sync file is unknown.")


class BaseAlgorithm(BasePlugin):
    """A base class for implementing an algorithm."""

    @classmethod
    @abc.abstractmethod
    def name(cls) -> str:
        """Return a name, which is used to represent this Algorithm in a dropdown in the GUI."""
        return "Basic Algorithm"

    @abc.abstractmethod
    def process_data(self, plot_data: Dict[str, PlotData]):  # noqa
        """Get labels from the data using an algorithm.

        This method applies an algorithm to the passed data.
        For example it could be a peak detection algorithm, which then for example creates one
        :class:`mad_gui.plot_tools.labels.BaseRegionLabel` between to consecutive peaks.
        This method can be accessed by the user by clicking the `Use algorithm` button in the GUI's sidebar.
        For more information and an example, see the part of `Implement an algorithm
        <https://mad-gui.readthedocs.io/en/latest/customization.html#implement-an-algorithm>`_
        in our online documentation.

        Parameters
        ----------
        plot_data
            A dictionary, where keys are the names of the plots in the GUI and the values are instances of
            :class:`mad_gui.models.local.PlotData`. These in turn keep the plotted sensor data, its sampling frequency,
            and the plotted annotations.

        Returns
        plot_data
            The adapted dictionary, where you have for example changed the data or annotations.
        """
        # df = some method that uses sensor data to get all the starts and ends for single activities / strides
        # return df
        raise NotImplementedError()


class BaseExporter(BasePlugin):
    """Export the plotted data and/or annotations."""

    @classmethod
    @abc.abstractmethod
    def name(cls) -> str:
        """Return a name, which is used to represent this Exporter in a dropdown in the GUI."""
        raise NotImplementedError()

    @abc.abstractmethod
    def process_data(self, global_data):
        """This method must be implemented by your plugin.

        Parameters
        ----------
        global_data
            The GUI's :class:`mad_gui.models.global_data.GlobalData` object, which is kept in
            :class:`mad_gui.windows.main.MainWindow`

        Returns
        -------
        Nothing
            Everything you want to to/save must be done inside your plugin.
        """
        # we ignore `unused-argument` because maybe some exporter needs this
        UserInformation(parent=self.parent).inform(
            "The functionality to calculate parameters from plotted data is not implemented for "
            "the chosen exporter / recording system."
        )
        raise NotImplementedError()



from mad_gui.components.dialogs.plugin_selection.plugin_selection_dialog import PluginSelectionDialog
from mad_gui.components.dialogs.plugin_selection.load_data_dialog import LoadDataDialog

__all__ = ["PluginSelectionDialog", "LoadDataDialog"]
